@@ -1,8 +1,9 @@
 module Main where
 
 import qualified Web.JWTTests
+import qualified Web.JWTTestsCompat
 import qualified Web.JWTInteropTests
-import qualified Web.Base64Tests
+import qualified Data.ByteString.ExtendedTests
 import qualified Data.Text.ExtendedTests
 import           Test.Tasty
 
@@ -12,8 +13,9 @@ main = defaultMain tests
 tests :: TestTree
 tests = testGroup "JWT Tests" [
                     Web.JWTTests.defaultTestGroup
+                  , Web.JWTTestsCompat.defaultTestGroup
                   , Web.JWTInteropTests.defaultTestGroup
-                  , Web.Base64Tests.defaultTestGroup
                   , Data.Text.ExtendedTests.defaultTestGroup
+                  , Data.ByteString.ExtendedTests.defaultTestGroup
                 ]
 
